require 'ams_AutoEye.rb'

msg = "You cannot load Auto Eye extension until SketchUp meets compatibility requirements of the plugin!"
raise(msg) unless defined?(AMS::Plugins::AutoEye)

AMS::Lib.require 'sketchup_window.rb'
AMS::Lib.require 'cursor.rb'

class AMS::Plugins::AutoEye

  def initialize
    @enabled = false
    @timer_id = nil
    @timestep = 1/64.0
    @speed = 35.degrees # Rotation speed in radians per second.
  end

  # Enable/Disable the tool.
  # @param [Boolean] state
  # @return [Boolean] true if state changed.
  def enabled=(state)
    state = state ? true : false
    return false if state == @enabled
    @enabled = state
    if @enabled
      AMS::Sketchup.add_observer(self)
    else
      AMS::Sketchup.remove_observer(self)
      swo_vp_onMouseEnter
    end
    true
  end

  # Determine whether the tool is enabled.
  # @return [Boolean]
  def enabled?
    @enabled
  end

  def swo_vp_onMouseEnter
    UI.stop_timer(@timer_id) if @timer_id
    @timer_id = nil
  end

  def swo_vp_onMouseLeave
    # Stop the timer just in case if it's still active.
    UI.stop_timer(@timer_id) if @timer_id
    # Create a new timer.
    @timer_id = UI.start_timer(@timestep, true){
      next unless AMS::Sketchup::MainWindow.active?
      p = AMS::Cursor.get_pos
      r = AMS::Sketchup::Viewport.rect
      next if p[0].between?(r[0], r[2]) and p[1].between?(r[1], r[3])
      c = AMS::Sketchup::Viewport.center
      d = Geom::Vector3d.new(p[0] - c[0], p[1] - c[1], 0).normalize
      s = @speed * @timestep
      sx = s * d.x
      sy = s * d.y
      t = Geom::Vector3d.new
      t.x = Math.sin(sx)
      t.y = Math.sin(-sy)
      t.z = (1 - (t.x**2 + t.y**2))**0.5
      view = Sketchup.active_model.active_view
      cam = view.camera
      tra = Geom::Transformation.new(cam.xaxis, cam.yaxis, cam.zaxis, cam.eye)
      cam.set(cam.eye, t.transform(tra), [0,0,1])
    }
  end

end # class AMS::Plugins::AutoEye

unless file_loaded?(__FILE__)
  auto_eye = AMS::Plugins::AutoEye.new
  auto_eye.enabled = true
  toolbar = UI::Toolbar.new 'AE'

  menu = UI.menu('View')

  cmd = UI::Command.new('cmd'){
    auto_eye.enabled = !auto_eye.enabled?
  }
  cmd.menu_text = cmd.tooltip = 'Auto Eye'
  cmd.status_bar_text = 'Enable/Disable auto eye.'
  cmd.set_validation_proc {
    auto_eye.enabled? ? MF_CHECKED : MF_UNCHECKED
  }
  cmd.small_icon = 'eye16.png'
  cmd.large_icon = 'eye24.png'

  toolbar.add_item(cmd)
  menu.add_item(cmd)

  toolbar.show

  file_loaded(__FILE__)
end
